#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <error.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/fb.h>

#define DEFAULT_FRAMEBUFFER	"/dev/fb0"
#define DEFAULT_FRAMEBUFFER_DEVFS	"/dev/fb/0"

static void
Usage(void)
{
	fprintf(stderr, "\nUsage: %s console [framebuffer]\n\n",
		program_invocation_short_name);
	exit(1);
}

int
main(int argc, const char *argv[])
{
	int     do_write = 0;
	int     fd;
	struct fb_con2fbmap map;

	switch (argc)
	{
		case 3:
			do_write = 1;
			map.framebuffer = atoi(argv[2]);
		case 2:
			map.console = atoi(argv[1]);
			break;
		default:
			Usage();
	}

	if (((fd = open(DEFAULT_FRAMEBUFFER_DEVFS, O_RDONLY)) < 0) &&
	    ((fd = open(DEFAULT_FRAMEBUFFER, O_RDONLY)) < 0))
		error(EXIT_FAILURE, errno, "open %s", DEFAULT_FRAMEBUFFER);

	if (do_write)
	{
		if (ioctl(fd, FBIOPUT_CON2FBMAP, &map))
			error(EXIT_FAILURE, errno, "ioctl FBIOPUT_CON2FBMAP");
	} else
	{
		if (ioctl(fd, FBIOGET_CON2FBMAP, &map))
			error(EXIT_FAILURE, errno, "ioctl FBIOGET_CON2FBMAP");
		printf("console %d is mapped to framebuffer %d\n",
		       map.console, map.framebuffer);
	}

	close(fd);
	return EXIT_SUCCESS;
}
